/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Map/mapcov.h>
#include <Map/mapfclass.h>
#include <Map/maplib.h>
#include <Map/mapprim.h>

#include <iostream.h>
#include <string.h>

/*****************************************************************************/

MapCoverage::MapCoverage (MapLibrary* lib, const char* name) : MapFiles(name, nil) {
    library(lib);
    _path = nil;
    _fclasses  = nil;
    _nfcls = -1;
    _edges = nil;
    _nedges = -1;
    _faces = nil;
    _nfaces = -1;
    _nodes = nil;
    _nnodes = -1;
}

MapCoverage::~MapCoverage () {
    int i;
    for (i=0; i<_nfcls; i++) 
	delete _fclasses[i];
    delete [] _fclasses;
    for (i=0; i<_nedges; i++) 
	delete _edges[i];
    delete [] _edges;
    for (i=0; i<_nfaces; i++) 
	delete _faces[i];
    delete [] _faces;
    for (i=0; i<_nnodes; i++) 
	delete _nodes[i];
    delete [] _nodes;
}

const char* MapCoverage::path() const {
    if (_library && !_path) {
	const char* libpath = _library->path();
	const int len = strlen(libpath) + strlen(_name) + 2;
        ((MapCoverage*)this)->_path = new char[len];
	char* covpath = ((MapCoverage*)this)->_path;
        strcpy( covpath, libpath );
        strcat( covpath, "/" );
        strcat( covpath, _name );
    }
    return _path;
}

int MapCoverage::nfcls() const{
    if (!initialized()) ((MapCoverage*)this)->init_feature_classes();

    return _nfcls;
}

MapFeatureClass* MapCoverage::feature_class(unsigned fclsnum) const{
    if (!initialized()) ((MapCoverage*)this)->init_feature_classes();
    
    if (fclsnum<_nfcls)
	return _fclasses[fclsnum];
    else
	return nil;
}
     
MapFeatureClass* MapCoverage::feature_class(const char* fclsname) const{
    if (!initialized()) ((MapCoverage*)this)->init_feature_classes();
    
    for (int i = 0; i < _nfcls; i++) {
	if (strcmp(fclsname,_fclasses[i]->name())==0)
	    return _fclasses[i];
    }
    return nil;
}
     
int MapCoverage::nedges() const{
    return _nedges;
}

MapEdge* MapCoverage::edge(unsigned edgenum) const{
    if (edgenum<_nedges)
	return _edges[edgenum];
    else
	return nil;
}
     
int MapCoverage::nfaces() const{
    return _nfaces;
}

MapFace* MapCoverage::face(unsigned facenum) const{
    if (facenum<_nfaces)
	return _faces[facenum];
    else
	return nil;
}
     
int MapCoverage::nnodes() const{
    return _nnodes;
}

MapNode* MapCoverage::node(unsigned nodenum) const{
    if (nodenum<_nnodes)
	return _nodes[nodenum];
    else
	return nil;
}
     
const char* MapCoverage::description() const { return "generic coverage description"; }

ostream& operator<< (ostream& out, const MapCoverage& cov) {
    out << "Coverage:  " << cov.name() << "\n";
    out << "Path:  " << cov.path() << "\n";
    out << "Description:  " << cov.description() << "\n";
    out << "Feature Classes:  " << cov.nfcls() << "\n";
    for (int i = 0; i < cov.nfcls(); i++) {
	out << cov.feature_class(i);
    }
    return out;
}

void MapCoverage::init_feature_classes() {
    if (initialized()) return;

    char **fclsnames = feature_class_names(_nfcls);
    if (_nfcls==0) return;

    _fclasses = new MapFeatureClass*[_nfcls];
    for (int i=0; i<_nfcls; i++) {
	_fclasses[i] = make_feature_class(fclsnames[i]);
	delete fclsnames[i];
    }
    delete [] fclsnames;
}

void MapCoverage::init_primitive_tables(int nedges, int nfaces, int nnodes) {
    int i;
    if (_edges) {
	for (i=0; i<_nedges; i++) 
	    delete _edges[i];
	delete _edges;
    }
    _nedges = nedges;
    _edges = new MapEdge*[_nedges];
    for (i=0; i<_nedges; i++) 
	_edges[i] = nil;

    if (_faces) {
	for (i=0; i<_nfaces; i++) 
	    delete _faces[i];
	delete _faces;
    }
    _nfaces = nfaces;
    _faces = new MapFace*[_nfaces];
    for (i=0; i<_nfaces; i++) 
	_faces[i] = nil;

    if (_nodes) {
	for (i=0; i<_nnodes; i++) 
	    delete _nodes[i];
	delete _nodes;
    }
    _nnodes = nnodes;
    _nodes = new MapNode*[_nnodes];
    for (i=0; i<_nnodes; i++) 
	_nodes[i] = nil;

}
